==> brumbrumrally_0.6.orig.tar.gz: brumbrumrally-0.6/src/Path.cpp <==
/*
 * Copyright 2012, 2013, 2016 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Path.h"
#include <istream>
#include <cstdlib>
#ifdef _WIN32
#include <windows.h>
#include <algorithm> // std::replace
#else
#include <sstream>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h> 
#endif


Path::Path()
:   path(".")
{
}

Path::Path(const std::string& path)
:   path(path)
{
	format();
}

Path Path::parent() const
{
	std::size_t pos = path.find_last_of('/');
	return Path(path.substr(0, pos));
}

Path Path::operator+(const std::string& filename) const
{
	return Path(path + '/' + filename);
}

Path& Path::operator+=(const std::string& filename)
{
	path += '/' + filename;
	format();
	return *this;
}

void Path::format()
{
	// Make sure path has the correct format!

#ifdef _WIN32
	// Replace backslashes with forward slashess. Backslash can be part 
	// of filenames in unix so we only do this convertion for windows.
	std::replace(path.begin(), path.end(), '\\', '/');
#endif
	
	// Remove duplicate consecutive forward slashes
	std::string::iterator it = path.begin();
	char preChar('\0');
	while (it != path.end())
	{
		if (preChar == '/' && *it == '/')
		{
			it = path.erase(it);
		}
		else
		{
			preChar = *it;
			++it;
		}
	}
	
	// make sure path is not empty
	if (path.empty())
	{
		path = ".";
	}
	// make sure the path is not ending with slash
	// useful if we want to add stuff to the filename
	else if (path[path.size() - 1] == '/')
	{
		path.erase(path.size() - 1);
	}
}

Path::operator const std::string&() const
{
	return path;
}

const char* Path::c_str() const
{
	return path.c_str();
}

bool Path::isDir() const
{
#ifdef _WIN32
	return GetFileAttributes(path.c_str()) & FILE_ATTRIBUTE_DIRECTORY;
#else
	struct stat st;
	return stat(path.c_str(), &st) == 0 && S_ISDIR(st.st_mode);
#endif
}

bool Path::isFile() const
{
#ifdef _WIN32
	return !isDir() && GetFileAttributes(path.c_str()) == INVALID_FILE_ATTRIBUTES;
#else
	struct stat st;
	return stat(path.c_str(), &st) == 0 && S_ISREG(st.st_mode);
#endif
}

bool Path::exist()
{
#ifdef _WIN32
	return !(GetFileAttributes(path.c_str()) & INVALID_FILE_ATTRIBUTES);
#else
	return (access(path.c_str(), F_OK) == 0);
#endif
}

bool Path::mkdir()
{
	Path prev = parent();
	if (!prev.exist() && prev.path.length() != path.length())
	{
		prev.mkdir();
	}
#ifdef _WIN32
	return CreateDirectory(path.c_str(), 0);
#else
	return ::mkdir(path.c_str(), 0700) == 0;
#endif
}

std::vector<std::string> Path::files() const
{
	std::vector<std::string> files;
#ifdef _WIN32
	WIN32_FIND_DATA wfd;
	HANDLE handle = FindFirstFile((path + "/*").c_str(), &wfd);
	if (handle != INVALID_HANDLE_VALUE)
	{
		do
		{
			if (wfd.cFileName[0] != '.')
			{
				files.push_back(wfd.cFileName);
			}
		} while (FindNextFile(handle, &wfd));
	}
#else
	DIR *dir = opendir(path.c_str());
	if (dir)
	{
		dirent *entry;
		while ((entry = readdir(dir)))
		{
			if (entry->d_name[0] != '.')
			{
				files.push_back(entry->d_name);
			}
		}
		closedir(dir);
	}
#endif
	return files;
}

static Path fetchExecutablePath()
{
	char path[256];
#ifdef _WIN32
	if (GetModuleFileName(0, path, sizeof(path)) == 0)
#else
	ssize_t length = readlink("/proc/self/exe", path, sizeof(path) - 1);
	if (length > 0)
	{
		path[length] = '\0';
	}
	else
#endif
	{
		return Path();
	}
	return Path(path).parent();
}

#ifndef _WIN32
static Path fetchHomePath()
{
	const char* env = std::getenv("HOME");
	if (env == 0) {
		return Path();
	}
	return Path(env);
}
#endif

Path fetchDataPath()
{
#ifndef _WIN32
	std::stringstream pathStream;
	const char* dataHome = std::getenv("XDG_DATA_HOME");
	if (dataHome && *dataHome != '\0')
	{
		pathStream << dataHome;
	}
	else
	{
		pathStream << (fetchHomePath() + ".local/share");
	}
	
	const char* dataDirs = std::getenv("XDG_DATA_DIRS");
	if (dataDirs && *dataDirs != '\0')
	{
		pathStream << ':' << dataDirs;
	}
	else
	{
		pathStream << ":/usr/local/share:/usr/share";
	}
	
	std::string strPath;
	while (std::getline(pathStream, strPath, ':'))
	{
		Path path(strPath + "/brumbrumrally");
		if (path.exist() && path.isDir())
		{
			return path;
		}
	}
#endif
	return fetchExecutablePath() + "/data";
}

const Path& dataPath()
{
	static Path path(fetchDataPath());
	return path;
}

Path fetchConfigPath()
{
	Path path;
#ifdef _WIN32
	const char* appdata = std::getenv("APPDATA");
	if (appdata && *appdata != '\0')
	{
		path = Path(appdata) + "/brumbrumrally";
	}
#else
	const char* configHome = std::getenv("XDG_CONFIG_HOME");
	if (configHome && *configHome != '\0')
	{
		path = Path(configHome) + "/brumbrumrally";
	}
	else
	{
		path = fetchHomePath() + "/.config/brumbrumrally";
	}
#endif
	if (!path.exist())
	{
		path.mkdir();
	}
	return path;
}

const Path& configPath()
{
	static Path path(fetchConfigPath());
	return path;
}

std::ostream& operator<<(std::ostream& s, const Path& p)
{
	s << p.c_str();
	return s;
}

==> brumbrumrally-0.7.tar.gz: brumbrumrally-0.7/src/Path.cpp <==
/*
 * Copyright 2012, 2013, 2016 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Path.h"
#include <istream>
#include <cstdlib>
#include <cstddef>
#ifdef _WIN32
#include <windows.h>
#include <algorithm> // std::replace
#else
#include <sstream>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h> 
#endif


Path::Path()
:   path(".")
{
}

Path::Path(const std::string& path)
:   path(path)
{
	format();
}

Path Path::parent() const
{
	std::size_t pos = path.find_last_of('/');
	return Path(path.substr(0, pos));
}

Path Path::operator+(const std::string& filename) const
{
	return Path(path + '/' + filename);
}

Path& Path::operator+=(const std::string& filename)
{
	path += '/' + filename;
	format();
	return *this;
}

void Path::format()
{
	// Make sure path has the correct format!

#ifdef _WIN32
	// Replace backslashes with forward slashes. Backslash can be part 
	// of filenames in unix so we only do this conversion for windows.
	std::replace(path.begin(), path.end(), '\\', '/');
#endif
	
	// Remove duplicate consecutive forward slashes.
	std::string::iterator it = path.begin();
	char preChar('\0');
	while (it != path.end())
	{
		if (preChar == '/' && *it == '/')
		{
			it = path.erase(it);
		}
		else
		{
			preChar = *it;
			++it;
		}
	}
	
	// Make sure path is not empty.
	if (path.empty())
	{
		path = ".";
	}
	// Make sure the path is not ending with slash
	// useful if we want to add stuff to the filename.
	else if (path[path.size() - 1] == '/')
	{
		path.erase(path.size() - 1);
	}
}

Path::operator const std::string&() const
{
	return path;
}

const char* Path::c_str() const
{
	return path.c_str();
}

bool Path::isDir() const
{
#ifdef _WIN32
	DWORD fileAttributes = GetFileAttributes(path.c_str());
	return fileAttributes != INVALID_FILE_ATTRIBUTES && fileAttributes & FILE_ATTRIBUTE_DIRECTORY;
#else
	struct stat st;
	return stat(path.c_str(), &st) == 0 && S_ISDIR(st.st_mode);
#endif
}

bool Path::isFile() const
{
#ifdef _WIN32
	DWORD fileAttributes = GetFileAttributes(path.c_str());
	return fileAttributes != INVALID_FILE_ATTRIBUTES && !(fileAttributes & FILE_ATTRIBUTE_DIRECTORY);
#else
	struct stat st;
	return stat(path.c_str(), &st) == 0 && S_ISREG(st.st_mode);
#endif
}

bool Path::exist()
{
#ifdef _WIN32
	return GetFileAttributes(path.c_str()) != INVALID_FILE_ATTRIBUTES;
#else
	return (access(path.c_str(), F_OK) == 0);
#endif
}

bool Path::mkdir()
{
	Path prev = parent();
	if (!prev.exist() && prev.path.length() != path.length())
	{
		prev.mkdir();
	}
#ifdef _WIN32
	return CreateDirectory(path.c_str(), 0);
#else
	return ::mkdir(path.c_str(), 0700) == 0;
#endif
}

std::vector<std::string> Path::files() const
{
	std::vector<std::string> files;
#ifdef _WIN32
	WIN32_FIND_DATA wfd;
	HANDLE handle = FindFirstFile((path + "/*").c_str(), &wfd);
	if (handle != INVALID_HANDLE_VALUE)
	{
		do
		{
			if (wfd.cFileName[0] != '.')
			{
				files.push_back(wfd.cFileName);
			}
		} while (FindNextFile(handle, &wfd));
	}
#else
	DIR *dir = opendir(path.c_str());
	if (dir)
	{
		dirent *entry;
		while ((entry = readdir(dir)))
		{
			if (entry->d_name[0] != '.')
			{
				files.push_back(entry->d_name);
			}
		}
		closedir(dir);
	}
#endif
	return files;
}

static Path fetchExecutablePath()
{
	char path[256];
#ifdef _WIN32
	if (GetModuleFileName(0, path, sizeof(path)) == 0)
#else
	ssize_t length = readlink("/proc/self/exe", path, sizeof(path) - 1);
	if (length > 0)
	{
		path[length] = '\0';
	}
	else
#endif
	{
		return Path();
	}
	return Path(path).parent();
}

#ifndef _WIN32
static Path fetchHomePath()
{
	const char* env = std::getenv("HOME");
	if (env == 0) {
		return Path();
	}
	return Path(env);
}
#endif

Path fetchDataPath()
{
#ifndef _WIN32
	std::stringstream pathStream;
	const char* dataHome = std::getenv("XDG_DATA_HOME");
	if (dataHome && *dataHome != '\0')
	{
		pathStream << dataHome;
	}
	else
	{
		pathStream << (fetchHomePath() + ".local/share");
	}
	
	const char* dataDirs = std::getenv("XDG_DATA_DIRS");
	if (dataDirs && *dataDirs != '\0')
	{
		pathStream << ':' << dataDirs;
	}
	else
	{
		pathStream << ":/usr/local/share:/usr/share";
	}
	
	std::string strPath;
	while (std::getline(pathStream, strPath, ':'))
	{
		Path path(strPath + "/brumbrumrally");
		if (path.exist() && path.isDir())
		{
			return path;
		}
	}
#endif
	return fetchExecutablePath() + "/data";
}

const Path& dataPath()
{
	static Path path(fetchDataPath());
	return path;
}

Path fetchConfigPath()
{
	Path path;
#ifdef _WIN32
	const char* appdata = std::getenv("APPDATA");
	if (appdata && *appdata != '\0')
	{
		path = Path(appdata) + "/brumbrumrally";
	}
#else
	const char* configHome = std::getenv("XDG_CONFIG_HOME");
	if (configHome && *configHome != '\0')
	{
		path = Path(configHome) + "/brumbrumrally";
	}
	else
	{
		path = fetchHomePath() + "/.config/brumbrumrally";
	}
#endif
	if (!path.exist())
	{
		path.mkdir();
	}
	return path;
}

const Path& configPath()
{
	static Path path(fetchConfigPath());
	return path;
}

std::ostream& operator<<(std::ostream& s, const Path& p)
{
	s << p.c_str();
	return s;
}
